/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** preferences
*/

#include "common.h"
#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#endif
#include <unistd.h>

#ifndef HAVE_STRSEP

char *strsep(char **stringp, const char *delim) {
	char *p,*q;

	if(!stringp || !*stringp || !**stringp || !delim)
		return NULL;
	p=*stringp;q=NULL;
	while( *p && !(q=(char*)index(delim,*p)) )
		p++;
	if(*p)
	{
		q=p;
		while( *q && index(delim,*q)) { *q=0;q++; }
		p=*stringp;
		*stringp=q;
		return p;
	} else {
		q=*stringp;
		*stringp=p;
		return q;
	}

}
#endif /* !HAVE_STRSEP*/


struct prfdata prf;
struct optionsdata options;

/* local variables */
static char *conffile = NULL;

#ifdef HAVE_GETOPT_LONG
static struct option longopts[] =
{
	{ "config", 1, 0, 'c' },
	{ "jid", 1, 0, 'j' },
	{ "resource", 1, 0, 'R' },
	{ "password", 1, 0, 'p' },
	{ "ask-for-password", 0, 0, 'P' },
	{ "option", 1, 0, 'o' },
	{ "register", 0, 0, 'r'},
	{ "help", 0, 0, 'h' },
	{ "version", 0, 0, 'V' }
};
#endif

static char *shortopts = "c:j:R:p:Po:rhV";

static void display_usage(void)
{
	printf("Usage: JabberX [OPTIONS] [JID] [PASS]\n"
		"The Cadillac of all text-based Jabber clients.\n\n"
		"  -c, --config=FILE       Configuration file\n"
		"  -j, --jid=JID           Your Jabber ID\n"
		"  -R, --resource=RESOURCE Your Jabber Resource\n"
		"  -p, --password=PASS     Your password\n"
		"  -P, --ask-for-password  Ask for password\n"
		"  -o, --option=OPT	   Parse Option\n"
		"     e.g.  -o 'COMPACT_LINES' or -o 'NO_COLORS=1'\n"
		"  -r, --register          Register this account\n"
		"  -h, --help              Display this help text and exit\n"
		"  -V, --version           Print version number and exit\n\n"
#ifndef HAVE_GETOPT_LONG
		"Sorry, long-options not available on your system....\n"
#endif
		"Report bugs to <palpa@jabber.org>.\n");

	exit(0);
}

typedef enum { aos_DEFAULT, aos_PARAM, aos_RC, aos_USER, aos_SERVER } aos;
typedef enum { aow_ALL, aow_NOFOREIGN} aow;

struct anoption 
{
	char *name;
	void *what;
	enum { aot_END, aot_ONOFF, aot_STRING } type ;
	aos set_by;
	aow allowed_for;
	void (*akt)(char*,void*,int);
};

static struct anoption availableOptions[] = 
{
        { "auto_chat",  	&options.auto_chat,		aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "popup_chat",  	&options.popup_chat,		aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "show_sent",  	&options.show_sent,		aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "hide_resource", 	&options.hide_resource,		aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "compact_lines", 	&options.compact_lines, 	aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "no_colors", 		&options.no_colors, 		aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "codeset",		&options.codeset, 		aot_STRING,	aos_DEFAULT, 	aow_NOFOREIGN,	optakt_codeset},
	{ "resend_presence", 	&options.resend_presence, 	aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "timestamps", 	&options.timestamps, 		aot_ONOFF,	aos_DEFAULT, 	aow_ALL,	NULL},
	{ "on_message", 	&options.on_message, 		aot_STRING,	aos_DEFAULT, 	aow_NOFOREIGN,	NULL},
	{ "on_roster", 		&options.on_roster, 		aot_STRING,	aos_DEFAULT, 	aow_NOFOREIGN,	NULL},
	{ "autoget_options",	&options.fetch_options,		aot_ONOFF,	aos_DEFAULT,	aow_NOFOREIGN,	NULL}, //Note: The NOFOREIGN is useless in this line, as long as there are not other measures.
	{ NULL, 		NULL, 				aot_END,	0, 		0,		NULL}
};

int parse_onoff_option(char *a)
{
	if (!a)
		return 0;
	switch( *a )
	{
		case 'o': case 'O':
			if( a[1] == 'N' || a[1] =='n' )
				return 1;
			else if( a[1] == 'F' || a[1] =='f' )
				return 2;
			else 
				return 3;
		/* on: */
		case '1': case 'y': case 'Y': case 'J': case 'j': case 'S': case 's':
			return 1;
		/* off: */
		case '0': case 'N': case 'n':
			return 2;
		/* toggle: */
		case 0: case 't': case 'T':
			return 0;
	}
	return 3;
}

int set_option(struct anoption *o,char *val,int gui)
{
	if( !o )
		return 0;
	switch( o->type ) {
		case aot_ONOFF:
			switch( parse_onoff_option(val) )
			{
				case 0:
					*(int*)o->what = !*(int*)o->what;
					break;
				case 1:
					*(int*)o->what = 1;
					break;
				case 2:
					*(int*)o->what = 0;
					break;
				case 3:
					if(gui)
					{
						gui_attr(GA_SYSMSG);
						gui_print("Unparseable: '%s'\n",val);
					} else 
						fprintf(stderr,"Unparseable: '%s'\n",val);
					return 0;
			}
			break;
		case aot_STRING:
			if( *(char**)o->what )
				free(*(char**)o->what);
			if( val && strlen(val) > 0 )
				*(char**)o->what = strdup(val);
			else
				*(char**)o->what = NULL;
			break;
		default:
#ifdef DEBUG
			fprintf(stderr,"What the hell?!\n");
#endif
			return 0;
	}
	if( o->akt ) 
		o->akt(o->name,o->what,gui);
	return 1;
}

void mem_print_option(char *dest,size_t size,struct anoption *o)
{
	if( !dest )
		return;
	if( !o )
	{
		*dest=0;
		return;
	}
	switch( o->type )
	{
		case aot_ONOFF:
			snprintf(dest,size,"%s",(*(int*)o->what)?"ON":"OFF");
			break;
		case aot_STRING:
			if( *(char**)o->what )
				snprintf(dest,size,"%s",*(char**)o->what);
			else
				*dest=0;
			break;
		case aot_END:
#ifdef DEBUG
			gui_print("Unprintable type of option!\n");
#endif
			*dest=0;
	}
}


void gui_print_option(struct anoption *o)
{
	if( !o )
		return;
	gui_attr(GA_MENU);
	switch( o->type )
	{
		case aot_ONOFF:
			gui_print("%-20s is %s\n",o->name,(*(int*)o->what)?"ON":"OFF");
			break;
		case aot_STRING:
			if( *(char**)o->what )
				gui_print("%-20s is '%s'\n",o->name,*(char**)o->what);
			else
				gui_print("%-20s is not set\n",o->name);
			break;
		default:
			gui_print("%s seems to have no printable type!?\n");
			break;
	}
}

int parse_option(int gui,aos set_by,char *arg)
{
	struct anoption *o;
	char buf[1000]; // as gui_print overwrites th conv-buffer, in which arg is...
	char *act,*val;
	int l;

	if(!arg || *arg == '\0' || strlen(arg)>=1000)
		return 0;
	strcpy(buf,arg);
	arg=buf;

	act = strsep(&arg, " \t");
	while( act ) 
	{
		if( act[(l=strlen(act))-1] == '\\' && arg )
		{
			act[l-1] = ' ';
			memmove(&act[l],arg,strlen(arg)+1);
			arg = &act[l];
			strsep(&arg, " \t");
			continue;
		}
		if((val = index(act,'=')))
		{
			l = val-act;
			val++;
		}
		else
		{
			l = strlen(act);
			val = index(act,0);
		}

		for( o = availableOptions ; o->type != aot_END ; o++ )
		{
			if( strncasecmp( act , o->name , l ) == 0 )
			{
			    if( !set_option( o , val , gui ) )
				    return 0;
			    o->set_by = set_by;
			    if( gui ) 
			    {
				    gui_print_option(o);
				    
			    }
			    act = NULL;
			    break;
			}
		}
		if( act ) 
		{
			if( gui )
			{
				gui_attr(GA_SYSMSG);
				gui_print("Option %s not found\n",act);
			}
			else
				fprintf(stderr,"Option %s not found\n",act);
			return 0;
		}
		if( arg )
			act = strsep(&arg, " \t");
		else
			act = NULL;
	}
	if ( gui && !options.compact_lines )
		gui_print("\n");	
	return 1;
}


int prf_setup(int argc, char *argv[])
{
	char *home, *s;
	iks *x,*y,*z;
	struct anoption *o;
	int c, i;
	int askpass = 0;

	/* defaults */
	memset(&prf, 0, sizeof(struct prfdata));
	memset(&options, 0, sizeof(struct optionsdata));
	prf.logfile = "./JabberX.log"; // FIXME: will be overwritten by later, even if not specified 

	/* command line arguments */
#ifdef HAVE_GETOPT_LONG
	while((c = getopt_long(argc, argv, shortopts, longopts, &i)) != -1)
#else
	while((c = getopt(argc, argv, shortopts)) != -1)
#endif
	{
		switch(c)
		{
		case 'h':
			display_usage();

		case 'V':
			printf("%s %s\n", PACKAGE, VERSION);
			exit(0);

		case 'c':
			conffile = optarg;
			break;

		case 'o':
			if(!parse_option(0,aos_PARAM,optarg))
				return 0;
			break;

		case 'j':
			prf.user = optarg;
			break;

		case 'p':
			prf.pass = optarg;
			break;

		case 'P':
			askpass = 1;
			break;
			
		case 'R':
			prf.resource = optarg;
			break;

		case 'r':
			net.regflag = 1;
			break;
		}
	}

	for(i=optind; i<argc; i++)
	{
		if(!prf.user) prf.user = argv[i];
		else if(!prf.pass) prf.pass = argv[i];
	}

	/* configuration file */
	if(!conffile)
	{
		home = getenv("HOME");
		if(home)
		{
			conffile = malloc(strlen(home) + 15);
			strcpy(conffile, home);
			strcat(conffile, "/.JabberXrc");
		}
		else
		{
			conffile = ".JabberXrc";
		}
	}

	x = iks_load(conffile);
	if(x)
	{
		if(!prf.user) prf.user = iks_strdup(iks_find_cdata(x, "jid"));
		if(!prf.resource) prf.resource = iks_strdup(iks_find_cdata(x, "resource"));
		if(!prf.pass && !askpass) prf.pass = iks_strdup(iks_find_cdata(x, "password"));
		if((y = iks_find(x,"options")))
		{
			for( o = availableOptions ; o->type != aot_END ; o++ )
			{
				if( o->set_by != aos_PARAM && (z = iks_find(y,o->name) ))
				{
					s = NULL;
					if( (z=iks_child(z)) )
						s = iks_cdata(z);

					set_option(o,s,0);
				}
			}


		}
		if(iks_find(x, "ask-password") && !prf.pass)
			askpass=1;
		prf.logfile = iks_strdup(iks_find_cdata(x, "logfile"));
	}
	iks_delete(x);
	if( askpass && prf.user)
		prf.pass = (char*)getpass("password:");
	if (!prf.resource) prf.resource = "JabberX";
	return 1;
}

void cmd_option(char *arg)
{
	struct anoption *o;
	
	if ( !arg || !*arg )
	{
		gui_attr(GA_MENU);
		gui_print("Options: (see /help option, when /help in implemented)\n");
		for( o = availableOptions ; o->type != aot_END ; o++ )
		{
			gui_print_option(o);
		}
	}	
	else
		parse_option(1,aos_USER,arg);
	return;
}

void cmd_option_save(char *arg)
{
    iks *x, *y, *z;
    struct anoption *o;
    char buffer[900];

    x = iks_make_iq(IKS_TYPE_SET, IKS_NS_PRIVATE);
    y = iks_find(x, "query");
    z = iks_insert(y, "jabberx");
    iks_insert_attrib(z, "xmlns", "jabberx:prefs");

    for( o = availableOptions ; o->type != aot_END ; o++ )
    {
	if( o->allowed_for == aow_ALL ) {
		mem_print_option(buffer,900,o);
		iks_insert_cdata(iks_insert(z, o->name),buffer, -1);
	}
    }

    iks_send(net.parser, x);
}

void cmd_option_load(char *arg)
{
    iks *x, *y, *z;

    x = iks_make_iq(IKS_TYPE_GET, IKS_NS_PRIVATE);
    y = iks_find(x, "query");
    z = iks_insert(y, "jabberx");
    iks_insert_attrib(z, "xmlns", "jabberx:prefs");

    iks_send(net.parser, x);
}

void parse_options_from_server(ikspak *pak)
{
    iks *x, *y;
    struct anoption *o;

    switch(pak->subtype)
    {
	    case IKS_TYPE_SET:
		    gui_attr(GA_SYSMSG);
		    gui_print("Saved options.\n");
		    if( !options.compact_lines )
			    gui_print("\n");
		    break;
	    case IKS_TYPE_RESULT:
		    x = iks_child(pak->query);
		    while(x)
		    {
			    if((iks_strcmp(iks_name(x), "jabberx") == 0) && (iks_strcmp(iks_find_attrib(x, "xmlns"), "jabberx:prefs") == 0))
			    {
				    y = iks_child(x);
				    while(y)
				    {
					    for( o = availableOptions ; o->type != aot_END ; o++ )
						    if( y->name && strcasecmp(y->name,o->name)==0 ) {
							    if( o->allowed_for == aow_ALL ) {
								    set_option(o,iks_cdata(iks_child(y)),1);
								    o->set_by = aos_SERVER;
							    }
						    }
					    y = iks_next(y);
				    }
			    }
			    x = iks_next(x);
		    }
		    cmd_option(NULL);
		    break;
	    default:
		    gui_print("Parse Save options failed\n");
		    if( !options.compact_lines )
			    gui_print("\n");
		    iks_delete(pak->x);
    }
}

