/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** presence
*/

#include "common.h"


void presence_parse(ikspak *pak)
{
	juser *u;
	iks *x;
	char *errnum;

	gui_attr(GA_BODY);
	u = roster_find(pak->from);
	if(u && u->name)
		gui_print("%s (%s) ", u->name, iks_id_print(pak->from));
	else
		gui_print("%s ", iks_id_print(pak->from));

	gui_attr(GA_HEADER);
	switch(pak->subtype)
	{
		case IKS_TYPE_ERROR:
			gui_attr(GA_SYSMSG);
			gui_print("reports error");
			x = iks_find(pak->x, "error");
			if(x)
			{
				errnum = iks_find_attrib(x, "code");
				if(errnum)
					gui_print("[%s]: ", errnum);
				else
					gui_print(": ");

				gui_attr(GA_BODY);
				gui_print("%s\n", iks_cdata(iks_child(x)));
			}
			else
			{
				gui_attr(GA_BODY);
				gui_print(": No error message!\n");
			}
			break;

		case IKS_TYPE_SUBSCRIBE:
			gui_print("wants to subscribe your presence.\n");
			break;

		case IKS_TYPE_SUBSCRIBED:
			gui_print("accepts your subscription.\n");
			break;

		case IKS_TYPE_UNSUBSCRIBE:
			gui_print("wants to unsubscribe from your presence.\n");
			break;

		case IKS_TYPE_UNSUBSCRIBED:
			gui_print("unsubscribes you from her presence.\n");
			break;
		default:
			gui_print("(unknown presence code)\n");
	}
	if( !options.compact_lines )
		gui_print("\n");
}


void cmd_presence(char *arg)
{
	iks *x;
	char *stat=NULL, *t;
	int show = IKS_SHOW_AVAILABLE;

	if(arg) {
	t = strtok(arg, " ");
	if(t)
	{
		if(strcasecmp(t, "off") == 0)
			show = IKS_SHOW_UNAVAILABLE;
		else if(strcasecmp(t, "away") == 0)
			show = IKS_SHOW_AWAY;
		else if(strcasecmp(t, "xa") == 0)
			show = IKS_SHOW_XA;
		else if(strcasecmp(t, "dnd") == 0)
			show = IKS_SHOW_DND;
		else if(strcasecmp(t, "chat") == 0)
			show = IKS_SHOW_CHAT;

		t = strtok(NULL, "");
		if(t) stat = t;
	}
	}

	if(show == IKS_SHOW_UNAVAILABLE)
		x = iks_make_pres(IKS_TYPE_UNAVAILABLE, 0, NULL, stat);
	else
		x = iks_make_pres(IKS_TYPE_AVAILABLE, show, NULL, stat);

	iks_send(net.parser, x);
	iks_delete(x);
}


void cmd_unsubscribe(char *arg)
{
	iks *x;

	if(!arg || *arg == '\0')
	{
		gui_attr(GA_NOTICE);
		gui_print("usage: un <jid>\n");
                if (!options.compact_lines)
                   gui_print ("\n");
		return;
	}

	x = iks_make_pres(IKS_TYPE_UNSUBSCRIBE, 0, arg, NULL);
	iks_send(net.parser, x);
	iks_delete(x);

	gui_attr(GA_NOTICE);
	gui_print("unsubscription request sent.\n");
        if (!options.compact_lines)
           gui_print ("\n");
}

void cmd_subscribe(char *arg)
{
	iks *x;

	if(!arg || *arg == '\0')
	{
		gui_attr(GA_NOTICE);
		gui_print("usage: su <jid>\n");
                if (!options.compact_lines)
                   gui_print ("\n");
		return;
	}

	x = iks_make_pres(IKS_TYPE_SUBSCRIBE, 0, arg, NULL);
	iks_send(net.parser, x);
	iks_delete(x);

	gui_attr(GA_NOTICE);
	gui_print("subscription request sent.\n");
        if (!options.compact_lines)
           gui_print ("\n");
}


void cmd_subscribed(char *arg)
{
	iks *x;

	if(!arg || *arg == '\0')
	{
		gui_attr(GA_NOTICE);
		gui_print("usage: sy <jid>\n");
                if (!options.compact_lines)
                    gui_print ("\n");
		return;
	}

	x = iks_make_pres(IKS_TYPE_SUBSCRIBED, 0, arg, NULL);
	iks_send(net.parser, x);
	iks_delete(x);
}
