/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** roster and presence stuff
*/

#include "common.h"

#define id_print IKS_ID_USER | IKS_ID_SERVER | (options.hide_resource?0:IKS_ID_RESOURCE)

static void roster_update_item(iks *x);
static jgroup *get_group(char *name);
static jgroup *is_group(char *name);
static void add_groupref(list **reflist, jgroup *group) ;
static int has_groupref(list **reflist, jgroup *group) ;
static void reset_grouprefs(list **reflist);

static list *contacts = NULL;
static list *groups = NULL;


void roster_fetch(void)
{
	iks *x;

	x = iks_make_iq(IKS_TYPE_GET, IKS_NS_ROSTER);
	iks_send(net.parser, x);
	iks_delete(x);
}


void roster_update(ikspak *pak)
{
	iks *x;
	juser *u;

	switch(pak->subtype)
	{
	case IKS_TYPE_RESULT:
		ui_xprint(UIF_TIMESTAMP|UIF_SETATTR|UIF_NEWLINE|UIF_EXTRANL,mainbuf,
				GA_SYSMSG,"Got roster.");
		x = iks_make_pres(0, 0, NULL, NULL);
		iks_send(net.parser, x);
		iks_delete(x);

	case IKS_TYPE_SET:
		x = iks_child(pak->query);
		while(x)
		{
			if(iks_strcmp(iks_name(x), "item") == 0)
				roster_update_item(x);
			x = iks_next(x);
		}
		iks_delete(pak->x);
		if( options.resend_presence ) {
			/* Check presence again (helpfull under some servers when switching client) - from Peter Toneby */
			u = list_first(&contacts);
			while(u) {
				x = iks_make_pres (IKS_TYPE_PROBE, IKS_SHOW_XA, iks_id_print (u->id), NULL);
				iks_send (net.parser, x);
				u = list_next(u);
				iks_delete (x);
			}
		}
		break;

	default:
		ui_xprint(UIF_TIMESTAMP|UIF_SETATTR|UIF_NEWLINE|UIF_EXTRANL,mainbuf,
				GA_SYSMSG,"parse roster failed");
		iks_delete(pak->x);
	}
}


static void roster_update_item(iks *x)
{
	juser *u;
	ikspool *p;
	iksid *id;
	char *tmp;
	iks *c,*cd;

	tmp = iks_find_attrib(x, "jid");
	if(!tmp) return;

	p = iks_pool_new(256);
	if(!p) return;
	id = iks_id_new(p, tmp);

	u = roster_find(id);
	if(u)
	{
		iks_pool_delete(p);
		p = u->p;
		reset_grouprefs(&u->groups);
	}
	else
	{
		u = iks_pool_alloc(p, sizeof(juser));
		if(!u) return;
		memset(u, 0, sizeof(juser));
		u->p = p;
		u->id = id;

		list_append(&contacts, u);
	}

	tmp = iks_find_attrib(x, "name");
	if(tmp)
	{
		if (u->name && strcmp(u->name, tmp))
		{
			ui_xprint(UIF_TIMESTAMP|UIF_SETATTR,mainbuf,GA_BODY,
				"%s [%s] is now known as %s.\n",
				u->name,
				iks_id_print(u->id),
				tmp);
		}
		u->name = iks_pool_strdup(p, tmp);
	} else {
		/* Too avoid NULL-references... */
		u->name = iks_pool_strdup(p, iks_id_print(u->id) );
	}

	c = iks_child(x);
	while( c ) {
		if( iks_type(c) == IKS_TAG && 0== strcasecmp(iks_name(c),"group") )
		{
			cd = iks_child(c);
			add_groupref(&u->groups,get_group(cd->cdata));
			
		}
		c = iks_next(c);
	}

	tmp = iks_find_attrib(x, "subscription");
	if(tmp)
	{
		if(strcmp(tmp, "none")==0)
			u->subs = JUSER_NONE;
		else if(strcmp(tmp, "to")==0)
			u->subs = JUSER_TO;
		else if(strcmp(tmp, "from")==0)
			u->subs = JUSER_FROM;
		else if(strcmp(tmp, "both")==0)
			u->subs = JUSER_BOTH;
		else if(strcmp(tmp, "remove")==0)
		{
			gui_xprint(UIF_NEWLINE|UIF_EXTRANL|UIF_SETATTR,GA_NOTICE,"Server removed item %s",iks_id_print(u->id));
			list_remove(&contacts, u);
			ui_xprint(UIF_TIMESTAMP|UIF_SETATTR,mainbuf,GA_BODY,
				"%s is now removed from roster.\n",
				iks_id_print(u->id));
		}
	}


}


void roster_update_presence(ikspak *pak)
{
	juser *u;
	iks *x;
	char *errnum,*status;

	if(pak->subtype == IKS_TYPE_ERROR)
	{
		ui_xprint(UIF_TIMESTAMP|UIF_SETATTR,mainbuf,
				GA_SYSMSG,"Presence error");
		if(pak->from)
		{
			gui_print(" from ");
			gui_attr(GA_BODY);
			u = roster_find(pak->from);
			if(u && u->name)
				gui_print("%s (%s) ", u->name, iks_id_printx(pak->from,id_print));
			else
				gui_print("%s ", iks_id_printx(pak->from,id_print));
			gui_attr(GA_SYSMSG);
		}
		x = iks_find(pak->x, "error");
		if(x)
		{
			errnum = iks_find_attrib(x, "code");
			if(errnum)
				gui_print("[%s]: ", errnum);
			else
				gui_print(": ");

			gui_attr(GA_BODY);
			gui_print("%s\n", iks_cdata(iks_child(x)));
		}
		else
		{
			gui_attr(GA_BODY);
			gui_print(": No error message!\n");
		}
		return;

	}

	status = iks_find_cdata(pak->x,"status");

	u = roster_find(pak->from);
	if(u) 
	{ 
		u->show = pak->show;
		if( u->status ) {
			// Where is the iks_pool_free ??
		}
		if( status )
			u->status = iks_pool_strdup(u->p,status);
		else
			u->status = NULL;
	}

	if(!u && gchat_pres(pak)) return;

	gui_attr(GA_BODY);
	if(u && u->name)
		ui_xprint(UIF_TIMESTAMP|UIF_SETATTR,mainbuf,GA_BODY,
				"%s (%s) ", u->name, iks_id_printx(pak->from,id_print));
	else
		ui_xprint(UIF_TIMESTAMP|UIF_SETATTR,mainbuf,GA_BODY,
				"%s ", iks_id_printx(pak->from,id_print));

	gui_attr(GA_HEADER);
	switch(pak->show)
	{
	case IKS_SHOW_UNAVAILABLE: gui_print("is unavailable"); break;
	case IKS_SHOW_AVAILABLE: gui_print("is available"); break;
	case IKS_SHOW_AWAY: gui_print("is away"); break;
	case IKS_SHOW_XA: gui_print("is xa"); break;
	case IKS_SHOW_DND: gui_print("is dnd"); break;
	case IKS_SHOW_CHAT: gui_print("is ready for chat"); break;
	}
	if( status ) 
		gui_print(": %s.\n",status);
	else
		gui_print(".\n");
	if(!options.compact_lines)
		gui_print("\n");
	on_exec(options.on_roster);

}

static void send_user(juser* u,char* jid, char* name)
{
	iks *x, *y, *z;
	jgroupref *r;

	x = iks_make_iq(IKS_TYPE_SET, "jabber:iq:roster");
	// Perhaps in iksemel?
	z = iks_find(x,"query");
	y = iks_insert(z,"item");
	iks_insert_attrib(y, "jid", (u)?iks_id_print(u->id):jid);
	iks_insert_attrib(y, "name", (name)?name:u->name);
	if( u && u->groups ) 
	{
		r = list_first(&u->groups);
		while( r )
		{
			z = iks_insert(y,"group");
			iks_insert_cdata(z,r->ref->name,strlen(r->ref->name));
			r = list_next(r);
		}
	}
	iks_send(net.parser, x);
	iks_delete(x);
}

juser *roster_find(iksid *id)
{
	juser *u;

	if(!contacts) return(NULL);

	u = list_first(&contacts);
	while(u)
	{
		if(iks_id_cmp(id, u->id) == 0) return(u);
		u = list_next(u);
	}

	u = list_first(&contacts);
	while(u)
	{
		if(iks_id_cmpx(id, u->id, IKS_ID_USER | IKS_ID_SERVER) == 0) return(u);
		u = list_next(u);
	}

	return NULL;
}


juser *roster_find_by_name(char *name, int isonline)
{
	juser *u;

	if(!name || !contacts) return(NULL);

	u = list_first(&contacts);
	while(u)
	{
		if((u->name && strcmp(name, u->name) == 0 && (u->show || !isonline)) || (u->id && strcmp(iks_id_printx(u->id,id_print),name)==0) )
		{
 			return (u);
		}
		u = list_next(u);
	}
	u = list_first(&contacts);
	while(u)
	{
		if((u->name && strcasecmp(name, u->name) == 0 && (u->show || !isonline))||(u->id && strcasecmp(iks_id_printx(u->id,id_print),name)==0) ) 
		{
 			return (u);
		}
		u = list_next(u);
	}

	return NULL;
}

static void print_id(juser *u)
{
	if (!u) return;
	if(u->name)
		gui_print("%s (%s)", u->name, iks_id_printx(u->id,id_print));
	else
		gui_print("%s", iks_id_printx(u->id,id_print));
}

static void print_presence(juser* u)
{
	if (!u) return;
	gui_attr(GA_HEADER);
	switch(u->subs)
	{
	case JUSER_NONE: gui_print("    "); break;
	case JUSER_TO: gui_print("To  "); break;
	case JUSER_FROM: gui_print("Frm "); break;
	case JUSER_BOTH: gui_print("Bth "); break;
	default: gui_print("? ");
	}

	switch(u->show)
	{
	case IKS_SHOW_UNAVAILABLE: gui_print("    "); break;
	case IKS_SHOW_AVAILABLE: gui_print("ON  "); break;
	case IKS_SHOW_CHAT: gui_print("Cht "); break;
	case IKS_SHOW_AWAY: gui_print("Awy "); break;
	case IKS_SHOW_XA: gui_print("XAw "); break;
	case IKS_SHOW_DND: gui_print("DND "); break;
	}

	gui_attr(GA_BODY);
	print_id(u);
	gui_print("\n");
}

jgroup *is_group(char *name) {
	jgroup *g;
	
	if( groups ) {
		g = list_first(&groups);
		while( g ) 
		{
			if( 0 == strcmp(g->name,name) ) 
			{
				return g;
			}
			g = list_next(g);
		}
	} 
	return NULL;
}

jgroup *get_group(char *name) {
	jgroup *g;
	ikspool *p;
	
	if( groups ) {
		g = list_first(&groups);
		if( g ) 
		{
			p = g->p;
		} else 
			p = NULL;
		while( g ) 
		{
			if( 0 == strcmp(g->name,name) ) 
			{
				return g;
			}
			g = list_next(g);
		}
	} 
	else 
		p = NULL;
	if( !p ) 
		p = iks_pool_new(256);
	if( !p ) 
		return NULL;
	g = iks_pool_alloc(p, sizeof(jgroup));
	if( !g )
		return NULL;
	g->name = iks_pool_strdup(p,name);
	g->p = p;
	list_append(&groups,g);
	return g;
}

int has_groupref(list **reflist, jgroup *group) {
	jgroupref *r;

	if( !*reflist || !list_first(reflist) || !group) {
		return 0;
	} else {
		r = list_first(reflist);
		while( r ) {
			if( group == r->ref ) 
				return 1;
			r = list_next(r);
		}
	}
	return 0;
}

void add_groupref(list **reflist, jgroup *group) {
	ikspool *p;
	jgroupref *r;

	if( !*reflist || !list_first(reflist) ) {
		p = iks_pool_new(256);
	} else {
		r = list_first(reflist);
		p = r->p;
		while( r ) {
			if( group == r->ref ) 
				return;
			r = list_next(r);
		}
	}
	r = iks_pool_alloc(p, sizeof(jgroupref));
	if(!r)
		return;
	r->ref = group;
	r->p = p;
	list_append(reflist,r);
}

void reset_grouprefs(list **reflist) {
	ikspool *p;
	jgroupref *r;
	
	if( *reflist ) {
		if( (r = list_first(reflist) ) && ( p = r->p ) )
			iks_pool_delete(p);
		list_remove_all(reflist);
		*reflist = NULL;
	}
}

void rm_groupref(list** reflist, jgroup *group) {
	ikspool *p;
	jgroupref *r, *rnew;
	list* newlist = NULL;

	if( !*reflist || !list_first(reflist) ) return;

	p = iks_pool_new(256);
	r = list_first(reflist);
	while( r )
	{
		if( group != r->ref ) 
		{
			rnew = iks_pool_alloc(p, sizeof(jgroupref));
			if (rnew)
			{
				rnew->ref = group;
				rnew->p = p;
				list_append(&newlist,rnew);
			}
		}
		r = list_next(r);
	}
	reset_grouprefs(reflist);
	*reflist = newlist;
}

void cmd_roster(char *arg)
{
	juser *u;
 	int showall = 0;
	int orphans = 0;
	jgroup *g = NULL;
 
	if(!contacts) return;

 	if (arg) {
		if (!strcmp(arg,"all"))
	 		showall = 1;
		else if (!strcmp(arg,"none"))
			orphans = 1;
		else
			g = is_group(arg);
	}

	u = list_first(&contacts);
	while(u)
	{
		if (orphans)
		{
			if (!u->groups) print_presence(u);
		}
		else if (g)
		{
			if (has_groupref(&u->groups,g)) print_presence(u);
		}
		else if (showall || u->show)
		{
			print_presence(u);
		}

		u = list_next(u);
	}
	gui_print("\n");
}

void cmd_listgroups(char* arg)
{
	juser* u;
	jgroup* g;
	jgroupref *r;
	char* jid = NULL;

	if(arg && *arg != '\0')
	{
		jid = strtok(arg, " ");
	}

	if(jid)
	{
		u = roster_find_by_name(jid, 1);
		if (!u) u = roster_find_by_name(jid, 0);
		if (!u)
		{
			iksid* id = iks_id_new(NULL,jid);
			u = roster_find(id);
			iks_pool_delete(id->p);
		}
		if (!u)
		{
			gui_attr(GA_NOTICE);
			gui_print("Not found in roster: %s\n", jid);
			gui_print("usage: group list [jid or nick]\n\n");
			return;
		}
		if( u && u->groups ) 
		{
			gui_attr(GA_HEADER);
			print_id(u);
			gui_print(" is member of:\n");
			gui_attr(GA_BODY);
			r = list_first(&u->groups);
			while( r )
			{
				gui_print("%s\n",r->ref->name);	
				r = list_next(r);
			}
		}
		else
		{
			gui_attr(GA_HEADER);
			print_id(u);
			gui_print(" has no group.\n");
		}
	}
	else if (groups)
	{
		gui_attr(GA_HEADER);
		gui_print("Groups available:\n");
		gui_attr(GA_BODY);
		for (g = list_first(&groups); g ; g = list_next(g))
		{
			gui_print("%s\n", g->name);
		}
	}
	else
	{
		gui_attr(GA_NOTICE);
		gui_print("No group available.\n");
	}

}

void cmd_addgroup(char* arg)
{
	juser* u;
	char* jid = NULL;
	char* newgrp = NULL;

	if(arg && *arg != '\0')
	{
		jid = strtok(arg, " ");
		if(jid) newgrp = strtok(NULL, "");
	}

	if(!jid || !newgrp)
	{
		gui_attr(GA_NOTICE);
		gui_print("usage: group add <jid or nick> <new group>\n\n");
		return;
	}

	if (jid)
	{
		u = roster_find_by_name(jid, 1);
		if (!u) u = roster_find_by_name(jid, 0);
		if (!u)
		{
			iksid* id = iks_id_new(NULL,jid);
			u = roster_find(id);
			iks_pool_delete(id->p);
		}
		if (!u)
		{
			gui_attr(GA_NOTICE);
			gui_print("Not found in roster: %s\n", jid);
			return;
		}
	}

	add_groupref(&u->groups,get_group(newgrp));
	send_user(u,jid,NULL);
}

void cmd_rmgroup(char* arg)
{
	juser* u;
	jgroup* g;
	char* jid = NULL;
	char* newgrp = NULL;

	if(arg && *arg != '\0')
	{
		jid = strtok(arg, " ");
		if(jid) newgrp = strtok(NULL, "");
	}

	if(!jid || !newgrp)
	{
		gui_attr(GA_NOTICE);
		gui_print("usage: group rm <jid or nick> <new group>\n\n");
		return;
	}
	
	g = is_group(newgrp);
	if (!g)
	{
		gui_attr(GA_NOTICE);
		gui_print("No group with this name: %s\n", newgrp);
		return;
	}

	if (jid)
	{
		u = roster_find_by_name(jid, 1);
		if (!u) u = roster_find_by_name(jid, 0);
		if (!u)
		{
			iksid* id = iks_id_new(NULL,jid);
			u = roster_find(id);
			iks_pool_delete(id->p);
		}
		if (!u)
		{
			gui_attr(GA_NOTICE);
			gui_print("Not found in roster: %s\n", jid);
			return;
		}
	}

	rm_groupref(&u->groups,g);
	send_user(u,jid,NULL);
}

void cmd_group(char* arg)
{
	char* token;
	if (arg)
	{
		if ((token = strchr(arg,' ')))
			*(token++) = '\0';
		if (!strcmp(arg, "add"))
		{
			cmd_addgroup(token);
			return;
		}
		if (!strcmp(arg, "rm"))
		{
			cmd_rmgroup(token);
			return;
		}
		if (!strcmp(arg, "list"))
		{
			cmd_listgroups(token);
			return;
		}
	}

	gui_attr(GA_NOTICE);
	gui_print("usage: group {rm|add|list} [options]\n");
	return;

}


/* Remove an user from the roster: */
void cmd_remove(char *arg)
{
	iks *x,*y,*z;
	juser *u;

	if(!arg || !*arg)
	{
		gui_attr(GA_NOTICE);
		gui_print("usage: remove <jid or nick>\n\n");
		return;
	}

	u = roster_find_by_name(arg,0);

	x = iks_make_iq(IKS_TYPE_SET, "jabber:iq:roster");
	z = iks_find(x,"query");
	y = iks_insert(z,"item");
	if( u ) {
		iks_insert_attrib(y, "jid", iks_id_print(u->id));
	} else {
		iks_insert_attrib(y, "jid", arg);
	}
	iks_insert_attrib(y, "subscription", "remove");
	iks_send(net.parser, x);
	iks_delete(x);
}

/* Set a nick or change it, perhaps in "r" but it is simpler this way,
   though the namespace is more poluted */
void cmd_name(char *arg)
{
	juser *u;
	char *jid = NULL,*newnick =NULL;

	if(arg && *arg != '\0')
	{
		jid = strtok(arg, " ");
		if(jid) newnick = strtok(NULL, "");
	}
	if(!jid || !newnick)
	{
		gui_attr(GA_NOTICE);
		gui_print("usage: name <jid or nick> <new nick>\n\n");
		return;
	}

	if (jid)
	{
		u = roster_find_by_name(jid, 1);
		if (!u) u = roster_find_by_name(jid, 0);
		if (!u)
		{
			iksid* id = iks_id_new(NULL,jid);
			u = roster_find(id);
			iks_pool_delete(id->p);
		}
		if (!u)
		{
			gui_attr(GA_NOTICE);
			gui_print("Not found in roster: %s\n", jid);
			/* return; */
		}
	}

	send_user(u,jid,newnick);
}

#define ITEM_BODY "This message contains roster items."

void cmd_senditem(char *arg)
{
        juser *u, *it;
	jgroupref *r;
        iks *m = iks_new("message");
        iks *x, *i;
        char *tokarg = strdup(arg);
        char *to, *name;

        if (arg && strlen(arg)) {

                  to = strtok(tokarg, " ");

                  if (!to)
                            goto usage;

                  u = roster_find_by_name(to, 1);
                  if (!u)
                            u = roster_find_by_name(to, 0);
	
                  if(u)
                            iks_insert_attrib(m, "to", iks_id_print(u->id));
                  else
                            iks_insert_attrib(m, "to", to);

                  iks_insert_cdata(iks_insert(m, "body"), ITEM_BODY, -1);

                  /* FIXME: Make a way for iksemel to do this. */

                  x = iks_insert(m, "x");
                  iks_insert_attrib(x, "xmlns", "jabber:x:roster");

                  while (name = strtok(NULL, " "))
                  {
                            it = roster_find_by_name(name, 0);

                            if (!it)
                            {
                                    gui_attr(GA_NOTICE);
                                    gui_print("'%s' not found in roster.\n", name);
                                    goto cleanup;
                            }

                            i = iks_insert(x, "item");
                            iks_insert_attrib(i, "jid", iks_id_print(it->id));

                            if (it->name)
                            {
                                    iks_insert_attrib(i, "name", it->name);
                            }

                            if (it->groups)
                            {
                                    for (r = list_first(&it->groups); r; r = list_next(r))
                                    {
                                              iks_insert_cdata(iks_insert(i, "group"), r->ref->name, -1);
                                    }
                            }
                  }

                  iks_send(net.parser, m);
                  gui_attr(GA_NOTICE);
                  gui_print("Roster items sent.\n");
                  if (!options.compact_lines)
                            gui_print ("\n");
                  goto cleanup;
        }

usage:
        gui_attr(GA_NOTICE);
        gui_print("Usage: senditem <jid> <name> <name> ...\n");
        if (!options.compact_lines)
                gui_print ("\n");
        
cleanup:
	iks_delete(m);
        free(tokarg);
}

